//
//        Yasuharu Okamoto, 2023.02.01
//
data {
    int n_items;
    int n_persons;
    int<lower = 3> K;        //   Number of categories is larger than 2
    int X[n_persons,n_items];
}
transformed data {
    vector[K-2] a_K;
    for (k in 1:(K-2)) {
        a_K[k] = 1.0;
    }
}
parameters {
    real F[n_persons];
    real mu[n_items];
    real<lower = 0.0> lmbd[n_items];
    real<lower = 0.0> sgm[n_items];
    simplex[K-2] theta;
}
transformed parameters {
    real C[K-1];
    simplex[K] p[n_persons,n_items];
    C[1] = 0.0;
    C[K-1] = 1.0;
    if (K > 3) {
        for (j in 2:(K-2)){
            C[j] = C[j-1] + theta[j-1];
        }
    }
    for (i in 1:n_persons) {
        for (j in 1:n_items) {
            p[i][j][1] = normal_cdf(C[1],
                                    mu[j] + lmbd[j] * F[i], sgm[j]);
            p[i][j][K] = 1.0 - normal_cdf(C[K-1],
                                    mu[j] + lmbd[j] * F[i], sgm[j]);
            for (k in 2:(K-1)) {
                p[i][j][k] = normal_cdf(C[k],
                                    mu[j] + lmbd[j] * F[i], sgm[j]) -
                             normal_cdf(C[k-1],
                                    mu[j] + lmbd[j] * F[i], sgm[j]);
            }
        }
    }
}
model {
    theta ~ dirichlet(a_K);
    for (i in 1:n_persons) {
        F[i] ~ normal(0.0, 1.0);
    }
    for (j in 1:n_items) {
        lmbd[j] ~ exponential(0.1); 
        mu[j] ~ normal(0.0, 10.0);
        sgm[j] ~ exponential(0.1);
    }
    for (i in 1:n_persons) {
        for (j in 1:n_items) {
            X[i][j] ~ categorical(p[i][j]);
        }
    }
}
